@@ -11,7 +11,7 @@ RcppExport SEXP frailtyLDTFP( SEXP nburn_, SEXP nsave_, SEXP nskip_, SEXP ndispl
 		SEXP tobs_, SEXP type_, SEXP xce_, SEXP xtf_, SEXP alpha_, SEXP betace_, 
     SEXP betatf_, SEXP sigma2_, SEXP y_, SEXP v_, SEXP blocki_, SEXP tau2_, SEXP W_, SEXP D_, SEXP maxL_,
     SEXP a0_, SEXP b0_, SEXP m0_, SEXP S0inv_, SEXP gprior_, SEXP a0sig_, SEXP b0sig_, 
-    SEXP a0tau_, SEXP b0tau_) {
+    SEXP a0tau_, SEXP b0tau_, SEXP win_, SEXP mm_) {
 	BEGIN_RCPP
   
 	// Transfer R variables into C++;
@@ -23,6 +23,8 @@ RcppExport SEXP frailtyLDTFP( SEXP nburn_, SEXP nsave_, SEXP nskip_, SEXP ndispl
   const IntegerVector type(type_);// nrec by 1;
   const arma::mat xce = as<mat>(xce_); // pce by nrec;
   const arma::mat xtf = as<mat>(xtf_); // ptf by nrec;
+  const double win = as<double>(win_);
+  const int mm = as<int>(mm_);
   const int pce = xce.n_rows;
   const int ptf = xtf.n_rows;
   const int nrec = xce.n_cols;
@@ -102,9 +104,9 @@ RcppExport SEXP frailtyLDTFP( SEXP nburn_, SEXP nsave_, SEXP nskip_, SEXP ndispl
   double logliko;
   double liminf, limsup, llim, rlim, gllim, grlim;
   double xx0, xx1, gxx0, gxx1;
-  double logy, uwork, win;
+  double logy, uwork;
   int i1, evali;
-  int JJ, KK, mm;
+  int JJ, KK;
   double rej;
   
 	////////////////////////////////////////////////////////////////////////
@@ -133,7 +135,6 @@ RcppExport SEXP frailtyLDTFP( SEXP nburn_, SEXP nsave_, SEXP nskip_, SEXP ndispl
 	////////////////////////////////////////////////////////////////////////
 	// Start MCMC
 	////////////////////////////////////////////////////////////////////////
-  win=1; mm=10;
 	for (int iscan=0; iscan<nscan; iscan++){
   	R_CheckUserInterrupt();
     //Rprintf( "iscan = %d\n", iscan );
